@@ -1,4 +1,4 @@
-/* $Id: mac_prj_generator.cpp 600896 2020-01-29 15:48:51Z gouriano $
+/* $Id: mac_prj_generator.cpp 670742 2023-07-31 17:44:17Z gouriano $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -264,6 +264,12 @@ void CMacProjectGenerator::Generate(const string& solution)
         if (!proj_link.empty()) {
             AddString( *build_phases, proj_link);
         }
+        // project post-build script phase
+        string proj_postbuild_script(
+            CreateProjectPostBuildScript(prj, prj_files, *dict_objects));
+        if (!proj_postbuild_script.empty()) {
+            AddString( *build_phases, proj_postbuild_script);
+        }
         // project copybin script phase
         string proj_copybin_script(
             CreateProjectCopyBinScript(prj, prj_files, *dict_objects));
@@ -782,6 +788,40 @@ string CMacProjectGenerator::CreateProjectCustomScriptPhase(
     return kEmptyStr;
 }
 
+string CMacProjectGenerator::CreateProjectPostBuildScript(
+    const CProjItem& prj, const CProjectFileCollector& prj_files,
+    CDict& dict_objects)
+{
+    string root = CDirEntry::DeleteTrailingPathSeparator( GetRelativePath(
+        CDirEntry::AddTrailingPathSeparator( CDirEntry::ConcatPath(
+            GetApp().GetProjectTreeInfo().m_Compilers,
+            GetApp().GetRegSettings().m_CompilersSubdir))));
+    string absroot = CDirEntry::ConcatPath(
+        GetApp().GetProjectTreeInfo().m_Compilers,
+        GetApp().GetRegSettings().m_CompilersSubdir);
+    string script;
+    script += "export BUILD_TREE_ROOT=" + root + "\n";
+    script += "export NCBI_VDB_LIBPATH=" + GetApp().GetSite().GetConfigureEntry("XCode_ThirdPartyVDBBasePath") + "\n";
+    if (CFile(absroot + "/postbuild.sh").Exists())
+    {
+        script +=  "\"$BUILD_TREE_ROOT/postbuild.sh\"";
+        string proj_script(   GetUUID());
+        CRef<CArray> inputs(  new CArray);
+        AddString( *inputs, "$(TARGET_BUILD_DIR)/$(FULL_PRODUCT_NAME)");
+        CRef<CArray> outputs( new CArray);
+        CRef<CDict> dict_script( AddDict( dict_objects, proj_script));
+        AddArray(  *dict_script, "files");
+        AddArray(  *dict_script, "inputPaths",  inputs);
+        AddArray(  *dict_script, "outputPaths", outputs);
+        AddString( *dict_script, "isa", "PBXShellScriptBuildPhase");
+        AddString( *dict_script, "shellPath", "/bin/sh");
+        AddString( *dict_script, "shellScript", script);
+        AddString( *dict_script, "showEnvVarsInLog", "0");
+        return proj_script;
+    }
+    return kEmptyStr;
+}
+
 string CMacProjectGenerator::CreateProjectCopyBinScript(
     const CProjItem& prj, const CProjectFileCollector& prj_files,
     CDict& dict_objects)
@@ -1140,6 +1180,19 @@ void CMacProjectGenerator::CreateBuildSettings(CDict& dict_cfg, const SConfigInf
     } else {
         AddString( *settings, "STANDARD_C_PLUS_PLUS_LIBRARY_TYPE", "static");
     }
+
+    const CMsvcMetaMakefile& metamake( GetApp().GetMetaMakefile());
+    string extra = metamake.GetCompilerOpt("PtbExtensions", cfg);
+    if (!extra.empty()) {
+        list<string> ex_list;
+        NStr::Split(extra, LIST_SEPARATOR, ex_list, NStr::fSplit_Tokenize);
+        for (const string& key : ex_list) {
+            string value = metamake.GetCompilerOpt(key,cfg);
+            if (!value.empty()) {
+                AddString( *settings, key, value);
+            }
+        }
+    }
 }
 
 void CMacProjectGenerator::CreateProjectBuildSettings(
@@ -1187,6 +1240,18 @@ void CMacProjectGenerator::CreateProjectBuildSettings(
         AddString( *settings, "INSTALL_PATH", proj_dir + libs_out_install);
         AddString( *settings, "OBJROOT", objroot);
 
+        string extra = metamake.GetLibrarianOpt("PtbExtensions", cfg);
+        if (!extra.empty()) {
+            list<string> ex_list;
+            NStr::Split(extra, LIST_SEPARATOR, ex_list, NStr::fSplit_Tokenize);
+            for (const string& key : ex_list) {
+                string value = metamake.GetLibrarianOpt(key,cfg);
+                if (!value.empty()) {
+                    AddString( *settings, key, value);
+                }
+            }
+        }
+
     } else if (prj.m_ProjType == CProjKey::eDll) {
 
         string bld_out(libs_out);
@@ -1206,6 +1271,18 @@ void CMacProjectGenerator::CreateProjectBuildSettings(
         AddArray( *settings, "LIBRARY_SEARCH_PATHS", lib_paths);
         AddString( *settings, "EXECUTABLE_PREFIX", "lib");
 
+        string extra = metamake.GetLinkerOpt("PtbExtensions_dylib", cfg);
+        if (!extra.empty()) {
+            list<string> ex_list;
+            NStr::Split(extra, LIST_SEPARATOR, ex_list, NStr::fSplit_Tokenize);
+            for (const string& key : ex_list) {
+                string value = metamake.GetLinkerOpt(key,cfg);
+                if (!value.empty()) {
+                    AddString( *settings, key, value);
+                }
+            }
+        }
+
     } else if (prj.m_ProjType == CProjKey::eApp) {
 
         AddLinkerSetting( *settings, cfg, "GCC_INLINES_ARE_PRIVATE_EXTERN");
@@ -1217,6 +1294,18 @@ void CMacProjectGenerator::CreateProjectBuildSettings(
         AddString( *settings, "OBJROOT", objroot);
 
         AddArray( *settings, "LIBRARY_SEARCH_PATHS", lib_paths);
+
+        string extra = metamake.GetLinkerOpt("PtbExtensions_exe", cfg);
+        if (!extra.empty()) {
+            list<string> ex_list;
+            NStr::Split(extra, LIST_SEPARATOR, ex_list, NStr::fSplit_Tokenize);
+            for (const string& key : ex_list) {
+                string value = metamake.GetLinkerOpt(key,cfg);
+                if (!value.empty()) {
+                    AddString( *settings, key, value);
+                }
+            }
+        }
     }
     if (prj.m_ProjType != CProjKey::eDataSpec && prj.m_ProjType != CProjKey::eMsvc) {
         AddString( *settings, "MACH_O_TYPE", GetMachOType(prj));
